@@ -122,8 +122,9 @@ on_window1_realize                     (GtkWidget       *widget,
                                         gpointer         user_data)
 {
   XWMHints wm_hints;
-  Atom wm_window_protocols[3];
+  Atom wm_window_protocols[3], atom_type_dock[1], atom_window_type;
   static gboolean initialized = FALSE;
+  gboolean is_dock = (gboolean) user_data;
   
   if (!initialized)
     {
@@ -135,11 +136,28 @@ on_window1_realize                     (GtkWidget       *widget,
   wm_hints.flags = InputHint;
   wm_hints.input = False;
   
+  gdk_error_trap_push ();
+
   XSetWMHints (GDK_WINDOW_XDISPLAY (widget->window),
 	       GDK_WINDOW_XWINDOW (widget->window), &wm_hints);
   
   XSetWMProtocols (GDK_WINDOW_XDISPLAY (widget->window),
 		   GDK_WINDOW_XWINDOW (widget->window), wm_window_protocols, 3);
+  
+  if (is_dock) {
+    gint w, h;
+    gtk_window_get_size (GTK_WINDOW (widget), &w, &h);
+
+    atom_window_type = gdk_x11_get_xatom_by_name ("_NET_WM_WINDOW_TYPE");
+    atom_type_dock[0] = gdk_x11_get_xatom_by_name ("_NET_WM_WINDOW_TYPE_DOCK");
+    XChangeProperty (GDK_WINDOW_XDISPLAY (widget->window), 
+		     GDK_WINDOW_XWINDOW (widget->window), 
+		     atom_window_type,
+		     XA_ATOM, 32, PropModeReplace,
+		     (guchar *) &atom_type_dock, 1);
+  }
+  gdk_error_trap_pop ();
+
   gtk_window_stick (GTK_WINDOW (widget));
 
   if (!gok_keyboard_xkb_select (GDK_WINDOW_XDISPLAY (widget->window)))
@@ -362,26 +380,10 @@ on_radiobuttonTypeSwitch_toggled       (GtkToggleButton *togglebutton,
 
 
 void
-on_radiobuttonTypeMouseButton_toggled  (GtkToggleButton *togglebutton,
+on_radiobuttonTypeValuator_toggled (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-	gok_page_actions_toggle_type_mousebutton (gtk_toggle_button_get_active (togglebutton));
-}
-
-
-void
-on_radiobuttonTypeDwell_toggled        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	gok_page_actions_toggle_type_dwell (gtk_toggle_button_get_active (togglebutton));
-}
-
-
-void
-on_radiobuttonTypeMousePointer_toggled (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	gok_page_actions_toggle_type_mousepointer (gtk_toggle_button_get_active (togglebutton));
+	gok_page_actions_toggle_type_valuator (gtk_toggle_button_get_active (togglebutton));
 }
 
 
@@ -521,62 +523,6 @@ on_spinSwitchDelay_changed             (GtkEditable     *editable,
 	gok_page_actions_set_rate (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(editable)));	
 }
 
-
-void
-on_radiobuttonMouseButton0_toggled     (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	if (gtk_toggle_button_get_active (togglebutton) == TRUE)
-	{
-		gok_page_actions_set_number(1);
-	}
-}
-
-
-void
-on_radiobuttonMouseButton1_toggled     (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	if (gtk_toggle_button_get_active (togglebutton) == TRUE)
-	{
-		gok_page_actions_set_number(2);
-	}
-}
-
-
-void
-on_radiobuttonMouseButton2_toggled     (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	if (gtk_toggle_button_get_active (togglebutton) == TRUE)
-	{
-		gok_page_actions_set_number(3);
-	}
-}
-
-
-void
-on_radiobuttonMouseButton3_toggled     (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	if (gtk_toggle_button_get_active (togglebutton) == TRUE)
-	{
-		gok_page_actions_set_number(4);
-	}
-}
-
-
-void
-on_radiobuttonMouseButton4_toggled     (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	if (gtk_toggle_button_get_active (togglebutton) == TRUE)
-	{
-		gok_page_actions_set_number(5);
-	}
-}
-
-
 void
 on_radiobuttonButtonPress_toggled      (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
@@ -609,19 +555,6 @@ on_radiobuttonButtonClick_toggled      (GtkToggleButton *togglebutton,
 	}
 }
 
-
-void
-on_radiobuttonButtonDoubleClick_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-	if (gtk_toggle_button_get_active (togglebutton) == TRUE)
-	{
-		gok_page_actions_set_state (ACTION_STATE_DOUBLECLICK);
-	}
-}
-
-
 void
 on_spinButtonDelay_changed             (GtkEditable     *editable,
                                         gpointer         user_data)
@@ -642,7 +575,7 @@ void
 on_checkKeyAverage_toggled             (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-	gok_page_actions_mousepointer_keyaverage (gtk_toggle_button_get_active (togglebutton));
+	gok_page_actions_pointer_keyaverage (gtk_toggle_button_get_active (togglebutton));
 }
 
 
@@ -915,6 +848,103 @@ on_notebook2_switch_page               (GtkNotebook     *notebook,
 }
 
 
+void
+on_action_type_notebook_change_current_page
+                                        (GtkNotebook     *notebook,
+                                        gint             offset,
+                                        gpointer         user_data)
+{
+
+}
 
 
+void
+on_2d_valuator_radiobutton_toggled     (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	/* we always use 2d for now; the 1-d axis selection is an RFE */
+}
+
+
+void
+on_1d_radiobutton_toggled              (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	/* TODO */
+}
+
+
+void
+on_axis_selection_spinbutton_changed   (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+	/* TODO */
+}
+
+
+void
+on_activate_on_enter_button_toggled    (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_actions_set_type (ACTION_TYPE_MOUSEPOINTER);
+}
+
+
+void
+on_activate_on_dwell_button_toggled    (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_actions_set_type (ACTION_TYPE_DWELL);
+}
+
+
+void
+on_activate_on_move_button_toggled     (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_actions_set_type (ACTION_TYPE_VALUECHANGE);
+}
+
+
+void
+on_pointer_delay_spinbutton_changed    (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+	gok_page_actions_set_rate (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(editable)));	
+}
+
+
+void
+on_key_averaging_button_toggled        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_actions_pointer_keyaverage (gtk_toggle_button_get_active (togglebutton));
+}
+
+
+void
+on_core_pointer_button_toggled         (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_actions_set_is_corepointer (
+		gtk_toggle_button_get_active (togglebutton));
+}
+
+
+void
+on_xinput_device_button_toggled        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_actions_set_is_corepointer (
+		!gtk_toggle_button_get_active (togglebutton));
+}
+
+
+void
+on_joystick_button_toggled             (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_actions_set_is_corepointer (
+		!gtk_toggle_button_get_active (togglebutton));
+}
 
