@@ -62,7 +62,9 @@ gint m_NumberPredictions;
 char NameAccessMethod [MAX_ACCESS_METHOD_NAME];
 gboolean m_bUseGtkPlusTheme;
 gboolean m_bDriveCorePointer = FALSE;
+gboolean m_bExpandWindow;
 gboolean m_bUseXkbKbd;
+gboolean m_eDockType;
 
 /* This GConfClient will be shared by all gconf_client
  * calls in this file.
@@ -99,6 +101,8 @@ void gok_data_initialize ()
 	m_bCommandPredictOn = FALSE;
 	m_NumberPredictions = 5;
 	m_bUseGtkPlusTheme = FALSE;
+	m_bExpandWindow = TRUE;
+	m_eDockType = GOK_DOCK_NONE;
 
 	/* This GConfClient will be shared by all gconf_client
 	 * calls in this file.
@@ -115,6 +119,19 @@ void gok_data_initialize ()
 	m_bSettingsChanged = FALSE;
 }
 
+static GokDockType
+gok_data_dock_type_from_string (gchar *string)
+{
+	if (!strcmp (string, "top")) {
+		return GOK_DOCK_TOP;
+	}
+	else if (!strcmp (string, "bottom")) {
+		return GOK_DOCK_BOTTOM;
+	}
+	else
+		return GOK_DOCK_NONE;
+}
+
 /**
  * gok_data_construct_setting:
  *
@@ -195,12 +212,17 @@ gboolean gok_data_read_settings (void)
 	gok_gconf_get_bool (gconf_client, GOK_GCONF_USE_GTKPLUS_THEME,
                                  &m_bUseGtkPlusTheme);
 
-	/*	gok_gconf_get_bool (gconf_client, GOK_GCONF_DRIVE_COREPOINTER,
-		&m_bDriveCorePointer);*/
+	gok_gconf_get_bool (gconf_client, GOK_GCONF_EXPAND,
+                                 &m_bExpandWindow);
 
 	gok_gconf_get_bool (gconf_client, GOK_GCONF_USE_XKB_KBD,
                                  &m_bUseXkbKbd);
 
+	gok_gconf_get_string (gconf_client, GOK_GCONF_DOCK_TYPE,
+			      &a_gchar);
+        m_eDockType = gok_data_dock_type_from_string (a_gchar);
+	g_free (a_gchar);
+
 	/* Read in settings for individual access methods */
 	access_method_dirs
 	    = gconf_client_all_dirs (gconf_client,
@@ -668,6 +690,41 @@ void gok_data_set_use_xkb_kbd (gboolean val)
 	m_bSettingsChanged = TRUE;
 }
 
+/**
+ * gok_data_get_dock_type:
+ *
+ * returns: A GokDockType indicating whether or not GOK's main window is
+ *          a "DOCK" type window, and whether it is anchored top or bottom.
+ */
+GokDockType gok_data_get_dock_type ()
+{
+	return m_eDockType;
+}
+
+/**
+ * gok_data_set_dock_type:
+ * @val: a GokDockType indicating whether GOK's main keyboard should be a "DOCK" window.
+ */
+void gok_data_set_is_dock (GokDockType val)
+{
+	m_eDockType = val;
+	gchar *typestring;
+	switch (val) {
+	case GOK_DOCK_TOP:
+	  typestring = "top";
+	  break;
+	case GOK_DOCK_BOTTOM:
+	  typestring = "bottom";
+	  break;
+	default:
+	  typestring = "";
+	  break;
+	}
+	gok_gconf_set_string (gconf_client, GOK_GCONF_DOCK_TYPE,
+			      typestring);
+	m_bSettingsChanged = TRUE;
+}
+
 /**
  * gok_data_get_key_height
  *
@@ -936,6 +993,30 @@ void gok_data_set_drive_corepointer (gboolean bTrueFalse)
 	*/
 }
 
+/**
+ * gok_data_get_drive_corepointer:
+ *
+ * Returns: TRUE if the core pointer should track the GOK valuators, 
+ *          FALSE if it should not.
+ */
+gboolean gok_data_get_expand ()
+{
+    return m_bExpandWindow;
+}
+
+/**
+ * gok_data_set_expand:
+ * @bTrueFalse: The flag setting whether GOK window should expand to fill
+ *      the width of the screen.
+ */
+void gok_data_set_expand (gboolean bTrueFalse)
+{
+	m_bExpandWindow= bTrueFalse;
+	gok_gconf_set_bool (gconf_client, GOK_GCONF_EXPAND,
+			    m_bExpandWindow);
+	m_bSettingsChanged = TRUE;
+}
+
 /**
  * gok_data_backup_settings:
  *
@@ -1071,3 +1152,15 @@ gboolean gok_data_restore_setting (GokSetting* pSetting)
 	return codeReturned;
 }
 
+gdouble
+gok_data_get_valuator_sensitivity (void)
+{
+	/* TODO: implement in gconf */
+	return 1.0;
+}
+
+void 
+gok_data_set_valuator_sensitivity (gdouble multiplier)
+{
+	/* TODO: implement in gconf */
+}
