@@ -34,6 +34,13 @@
 #include "gok-branchback-stack.h"
 #include "gok-modifier-keymasks.h"
 
+#define XML_STRING_SIZE_STOCK ((const xmlChar *) "stock")
+#define XML_STRING_SIZE_FIXED ((const xmlChar *) "fixed")
+#define XML_STRING_SIZE_FIT ((const xmlChar *) "fit")
+#define XML_STRING_SIZE_FILL ((const xmlChar *) "fill")
+
+#define xmlStrPrefix(a, b) (!xmlStrncmp((a), (b), xmlStrlen(b)))
+
 gchar m_TextEmpty[]="";
 
 char m_label[XkbKeyNameLength + 1];
@@ -288,7 +295,9 @@ gok_key_get_default_border_height (GokKey *pKey)
                 GTK_WIDGET (pKey->pButton)->style->ythickness;
 }
 
-static gchar *
+/**
+ **/
+gchar *
 gok_key_modifier_for_keycode (Display *display, XkbDescPtr xkb, int keycode)
 {
 	gint group = gok_key_get_effective_group ();
@@ -318,6 +327,27 @@ gok_key_set_effective_group (gint group)
 	gok_keyboard_update_labels ();
 }
 
+void
+gok_keyimage_set_size_from_spec (GokKeyImage *keyimage, gchar *sizespec)
+{
+	if (xmlStrPrefix (sizespec, XML_STRING_SIZE_STOCK)) {
+		keyimage->type = IMAGE_TYPE_STOCK;
+	}
+	else if (xmlStrPrefix (sizespec, XML_STRING_SIZE_FIXED)) {
+		int w, h;
+		keyimage->type = IMAGE_TYPE_FIXED;
+		if (sscanf (sizespec, "%*6c%d,%d", &w, &h) == 2) {
+			keyimage->w = w;
+			keyimage->h = h;
+		}
+		fprintf (stderr, "image size: %d,%d\n", keyimage->w, keyimage->h);
+	}
+	else if (xmlStrPrefix (sizespec, XML_STRING_SIZE_FIT))
+		keyimage->type = IMAGE_TYPE_FIT;
+	else if (xmlStrPrefix (sizespec, XML_STRING_SIZE_FILL))
+		keyimage->type = IMAGE_TYPE_FILL;
+
+}
 
 GokKey *
 gok_key_from_xkb_key (GokKey *prevKey, GokKeyboard *pKeyboard, Display *display, XkbGeometryPtr pGeom, XkbRowPtr pRow, XkbSectionPtr pXkbSection, XkbKeyPtr keyp, int section, int row, int col)
@@ -332,6 +362,7 @@ gok_key_from_xkb_key (GokKey *prevKey, GokKeyboard *pKeyboard, Display *display,
   gboolean is_modifier = FALSE;
   unsigned int numlock_mask = gok_key_get_numlock_mask (display);
 
+  pKey->has_text = TRUE; /* TODO: handle non-textual keyboard keys! */
   pKey->Top = row;
   pBounds = &pGeom->shapes[keyp->shape_ndx].bounds;
   pKey->Bottom = row + MAX (1, (pBounds->y2 - pBounds->y1)/(pBounds->x2 - pBounds->x1));
@@ -438,7 +469,9 @@ GokKey* gok_key_new (GokKey* pKeyPrevious, GokKey* pKeyNext, GokKeyboard* pKeybo
 	}
 
 	/* initialize the data members of the structure */
-	pgok_key_new->ImageCode = KEYDISPLAY_TEXT;
+	pgok_key_new->has_image = FALSE;
+	pgok_key_new->has_text = TRUE;
+	pgok_key_new->pImage = NULL;
 	pgok_key_new->Type = KEYTYPE_NORMAL;
 	pgok_key_new->Style = KEYTYPE_NORMAL;
 	pgok_key_new->pLabel = NULL;
@@ -490,12 +523,23 @@ void gok_key_delete (GokKey* pKey, GokKeyboard* pKeyboard, gboolean bDeleteButto
 	/* Don't call gok_spy_free on the NodeAccessible since this will be done by the keyboard. */
 	GokKeyLabel* pLabel;
 	GokKeyLabel* pLabelTemp;
+	GokKeyImage* pImage;
+	GokKeyImage* pImageTemp;
 	
 	if (pKey == NULL)
 	{
 		return;
 	}
 
+	/* delete all the key's images */
+	pImage = pKey->pImage;
+	while (pImage != NULL)
+	{
+		pImageTemp = pImage;
+		pImage = pImage->pImageNext;
+		gok_keyimage_delete (pImageTemp);
+	}
+
 	/* if this key is flashing then turn off the flashing */
 	if (gok_feedback_get_key_flashing() == pKey)
 	{
@@ -567,6 +611,8 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 	g_assert (pKey != NULL);
 	g_assert (pNode != NULL);
 
+	pKey->has_text = FALSE;
+
 	/* type of key (normal, branch etc.) */
 	pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "type");
 	if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"normal") == 0)
@@ -745,6 +791,19 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 		if (xmlStrcmp (pNodeKeyChild->name, (const xmlChar *)"label") == 0)
 		{
 			gok_keylabel_new (pKey, (char *)xmlNodeGetContent (pNodeKeyChild), (char *)xmlGetProp (pNodeKeyChild, (const xmlChar *) "modifier"));
+			pKey->has_text = TRUE;
+		}
+
+		/* key image */
+		if (xmlStrcmp (pNodeKeyChild->name, (const xmlChar *)"image") == 0)
+		{
+			/* TODO: support href as well as local files ? */
+			gchar *filename = xmlGetProp (pNodeKeyChild, (const xmlChar *) "source");
+			gchar *sizespec = xmlGetProp (pNodeKeyChild, (const xmlChar *) "type");
+			gok_keyimage_new (pKey, filename);
+			if (sizespec != NULL) 
+				gok_keyimage_set_size_from_spec (pKey->pImage, sizespec);
+			pKey->has_image = TRUE;
 		}
 
 		/* output */
@@ -976,6 +1035,7 @@ int gok_key_get_label_length (GokKey* pKey)
 	GtkLabel* pLabel;
 
 	if ((pKey->pButton == NULL) ||
+	    (GOK_BUTTON (pKey->pButton)->pLabel == NULL) ||
 		(strlen (gok_key_get_label (pKey)) == 0))
 	{
 		return 0;
@@ -1005,12 +1065,14 @@ int gok_key_get_label_height (GokKey* pKey)
 	}
 	
 	pLabel = GTK_LABEL(((GokButton*)pKey->pButton)->pLabel);
-	g_assert (pLabel != NULL);
 	
-	pPangoLayout = gtk_label_get_layout (pLabel);
-	pango_layout_get_pixel_extents (pPangoLayout, &rectInk, &rectLogical);
-	
-	return rectLogical.y + MAX (rectLogical.height, rectInk.height); 
+	if (pLabel != NULL) {
+		pPangoLayout = gtk_label_get_layout (pLabel);
+		pango_layout_get_pixel_extents (pPangoLayout, &rectInk, &rectLogical);
+		return rectLogical.y + MAX (rectLogical.height, rectInk.height); 
+	}
+	else
+		return 0;
 }
 
 /**
@@ -1022,7 +1084,7 @@ int gok_key_get_label_height (GokKey* pKey)
 void gok_key_update_label (GokKey* pKey)
 {
 	gchar* pNewLabelText;
-	const gchar* pOldLabelText;
+	const gchar* pOldLabelText = NULL;
 	GtkWidget* pButtonLabel;
 	
 	if (pKey->pButton == NULL)
@@ -1035,7 +1097,9 @@ void gok_key_update_label (GokKey* pKey)
 	
 	/* get the current text displayed on the key */
 	pButtonLabel = ((GokButton*)pKey->pButton)->pLabel;
-	pOldLabelText = gtk_label_get_text (GTK_LABEL(pButtonLabel));
+	if (pButtonLabel != NULL) {
+		pOldLabelText = gtk_label_get_text (GTK_LABEL(pButtonLabel));
+	}
 	if (pOldLabelText == NULL)
 	{
 		return;
@@ -1070,6 +1134,63 @@ void gok_key_update_label (GokKey* pKey)
 	}
 }
 
+/**
+* gok_key_get_image
+* @pKey: Pointer to the key that you want the image for.
+*
+* Returns: A pointer to the key's GokKeyImage, or NULL if the key has no images.
+**/
+GokKeyImage* 
+gok_key_get_image (GokKey* pKey)
+{
+	return pKey->pImage;
+}
+
+/**
+* gok_key_create_image_widget
+* @pKey: Pointer to the key that you want the image for.
+*
+* Returns: A GtkWidget pointer to a newly-created GtkImage, NULL if no image could be created.
+**/
+GtkWidget* 
+gok_key_create_image_widget (GokKey* pKey)
+{
+	GtkWidget *image = NULL;
+	if (pKey->pImage != NULL) {
+		/* FIT and FILL aren't yet implemented, since they are dynamic */
+		switch (pKey->pImage->type) {
+		case IMAGE_TYPE_STOCK:
+			image = gtk_image_new_from_stock (pKey->pImage->Filename,
+							  pKey->pImage->stock_size);
+			break;
+		case IMAGE_TYPE_FIXED:
+		default:
+			fprintf (stderr, "scaled image %d,%d\n", 
+				 pKey->pImage->w, pKey->pImage->h);
+			if ((pKey->pImage->w == -1) && (pKey->pImage->h == -1)) {
+				image = gtk_image_new_from_file (
+					gok_key_get_image_filename (pKey));
+			}
+			else {
+				GdkPixbuf *scaled = NULL, *pixbuf;
+				pixbuf = gdk_pixbuf_new_from_file (
+					gok_key_get_image_filename (pKey), NULL);
+				if (pixbuf)
+					scaled = gdk_pixbuf_scale_simple (
+						pixbuf, pKey->pImage->w,
+						pKey->pImage->h, 
+						GDK_INTERP_BILINEAR);
+				g_object_unref (pixbuf);
+				if (scaled)
+					image = gtk_image_new_from_pixbuf (scaled);
+				g_object_unref (scaled);
+			}
+			break;
+		}
+	}
+	return image;
+}
+
 /**
 * gok_key_get_label
 * @pKey: Pointer to the key that you want the label for.
@@ -1331,8 +1452,8 @@ void gok_key_set_button_name (GokKey* pKey)
 	gtk_widget_set_name (pKey->pButton, styleButton); 
 
 	/* set 'name' of the button text */
-	g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-	gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, styleText);
+	if (((GokButton*)pKey->pButton)->pLabel != NULL)
+		gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, styleText);
 }
 
 
@@ -1371,6 +1492,7 @@ gok_key_update_toggle_state (GokKey *pKey)
 **/
 int gok_key_calculate_font_size (GokKey* pKey, gboolean bWidth, gboolean bHeight)
 {
+	/* TODO: account for any icons/images on this key! */
 	gint sizeFontWidth;
 	gint sizeFontHeight;
 	gint sizeFontReturned;
@@ -1566,7 +1688,8 @@ void gok_key_set_font_size (GokKey* pKey, gint Size)
 	{
 		/* add the Pango atrribute list to the key's label */
 		pButton = (GokButton*)pKey->pButton;
-		gtk_label_set_attributes (GTK_LABEL(pButton->pLabel), pPangoAttrs);
+		if (pButton->pLabel)
+			gtk_label_set_attributes (GTK_LABEL(pButton->pLabel), pPangoAttrs);
 	}
 	else
 	{
@@ -1790,3 +1913,81 @@ gboolean gok_key_contains_point (GokKey* pKey, gint x, gint y)
 	return FALSE;
 }
 
+/**
+* gok_keyimage_new
+* @pKey: Pointer to the key that gets the new image.
+* @pFilename: Filename containing the image.
+*
+* Allocates memory for a new key image and initialises the
+* GokKeyImage structure.  Returns a pointer to the new key image,
+* NULL if it can't be created.  Add this image to a key so it will be
+* deleted when the key is deleted.
+*
+* returns: A pointer to the new key image, NULL if it wasn't created.
+**/
+GokKeyImage*
+gok_keyimage_new (GokKey* pKey, gchar* pFilename)
+{
+	GokKeyImage* pNewImage;
+	GokKeyImage* pKeyImage;
+
+	pNewImage = (GokKeyImage*) g_malloc (sizeof (GokKeyImage));
+
+	pNewImage->Filename = NULL;
+	pNewImage->pImageNext = NULL;
+        /* "unspecified", i.e. scalable or context-dependent */
+	pNewImage->w = pNewImage->h = -1; 
+	
+	if (pFilename != NULL)
+	{
+		pNewImage->Filename = (gchar*) g_malloc (strlen (pFilename) + 1);
+		strcpy (pNewImage->Filename, pFilename);
+	}
+	
+	if (pKey != NULL)
+	{
+		pKeyImage = pKey->pImage;
+		if (pKeyImage == NULL)
+		{
+			pKey->pImage = pNewImage;
+		}
+		else
+		{
+			while (pKeyImage->pImageNext != NULL)
+			{
+				pKeyImage = pKeyImage->pImageNext;
+			}
+			pKeyImage->pImageNext = pNewImage;
+		}
+		pKey->has_image = TRUE;
+	}
+	
+	return pNewImage;
+	}
+	 
+/**
+* gok_keyimage_delete
+* @pKeyImage: Pointer to the key image that will be deleted.
+**/
+void
+gok_keyimage_delete (GokKeyImage* pKeyImage)
+{
+	if (pKeyImage != NULL)
+	{
+		if (pKeyImage->Filename != NULL)
+		{
+			g_free (pKeyImage->Filename);
+		}
+		g_free (pKeyImage);
+	}
+}
+
+gchar*
+gok_key_get_image_filename (GokKey* pKey)
+{
+	g_assert (pKey != NULL);
+	
+	/* this needs to get the filename for the normal modifier key */
+	return pKey->pImage->Filename;
+}
+
