@@ -154,7 +154,7 @@ gok_keyboard_notify_xkb_event (XkbEvent *event)
 				 sevent->group);
 			gok_key_set_effective_group (sevent->group);
 		}
-	}
+	} 
 	/* 
 	 * TODO: change GOK's shift state notification 
 	 * to use XKB instead of at-spi ?
@@ -1268,22 +1268,20 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 		if (pKey->pButton == NULL)
 		{
 			/* create a new GOK button */
-			pNewButton = gok_button_new_with_label (gok_key_get_label (pKey)); 
-/*
-			pBoxLabel = gtk_hbox_new (FALSE, 0);
-			gtk_container_set_border_width (GTK_CONTAINER (pBoxLabel), 2);
-
-			pStyle = gtk_widget_get_style (pNewButton);
-			pPixmap = gdk_pixmap_create_from_xpm (pNewButton->window, &mask,
-					 &pStyle->bg[GTK_STATE_NORMAL],
-					"test.xmp");
-			pPixmapwid = gtk_pixmap_new (pPixmap, mask);
-
-			pLabel = gtk_label_new (pKey->Label);
-			gtk_box_pack_start (GTK_BOX (pBoxLabel), pPixmapwid, FALSE, FALSE, 3);
-			gtk_widget_show(pPixmapwid);
-			gtk_widget_show (pLabel);
-*/
+			if (pKey->has_text)
+			{
+				pNewButton = gok_button_new_with_label (gok_key_get_label (pKey)); 
+				if (pKey->has_image)
+					gok_button_set_image (GOK_BUTTON (pNewButton), 
+							      GTK_IMAGE (gok_key_create_image_widget (pKey)));
+			}
+			else if (pKey->has_image)
+			{
+				pNewButton = gok_button_new_with_image (gok_key_create_image_widget (pKey));
+			}
+			else {
+			        pNewButton = g_object_new (GOK_TYPE_BUTTON, NULL);
+			}
 
 			/* associate the button with the key */
 			pKey->pButton = pNewButton;
@@ -1394,7 +1392,8 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 	}
 	
 	if (((widthWindow + frameX) > widthMax) || 
-	    (pKeyboard->LayoutType == KEYBOARD_LAYOUT_FITWINDOW))
+	    (pKeyboard->LayoutType == KEYBOARD_LAYOUT_FITWINDOW) ||
+	    gok_data_get_expand ())
 	{
 		/* change the key width (for this keyboard) to fit within the screen */
 		widthKeyTemp = gok_keyboard_get_keywidth_for_window (widthMax - (frameX * 2), pKeyboard);
@@ -1854,29 +1853,38 @@ gboolean gok_keyboard_branch_gui_actions (AccessibleNode* pNodeAccessible)
 	g_assert(pNodeAccessible->paccessible != NULL);
 	
 	/* 
-	 * just in case: before we do any actions check to see if this object has interesting children 
-	 * and if so, branch to them (before performing the action on the parent).
+	 * just in case: before we do any actions check to see if this object has 
+	 * interesting children and if so, branch to them instead.
 	 *
 	 */
+	gok_log("checking for children with interesting roles");
 	for (i=0; i< NUM_INTERESTING_ROLES; i++)
 	{
 		if (gok_spy_has_child (pNodeAccessible->paccessible, m_InterestingRole[i]) == TRUE)
 		{
-			gok_log_x("The current object being activated has interesting children!");
+			gok_log_x("Abort action. Branching to child(ren)!");
+			gok_log_leave();
 			return gok_keyboard_branch_gui(pNodeAccessible, m_InterestingRole[i]);
 		}
 	}
+	gok_log("no children with interesting roles");
 
-	/* TODO: build a keyboard if more than one action available (do this before above branch to children!)*/
-	/* for now just perform the first action */
-	gok_log ("this function not fully implemented - simply performing first action.");
+	/* possible TODO: build a keyboard if more than one action available 
+	(and do this before above branch to children!)*/
 	paaction = Accessible_getAction(pNodeAccessible->paccessible);
 	gok_spy_accessible_implicit_ref(paaction);
 	if (paaction != NULL)
 	{
+		/* for now just perform the first action 
+		*** warning repeated code block*/
+		{
+		Accessible_isAction(paaction);
 		AccessibleAction_doAction(paaction, 0);
 		gok_spy_accessible_unref(paaction);
-		/* now branch back to most recent *static* keyboard */
+		}
+
+		/* no interesting children roles found so now branch back to most 
+		recent *static* keyboard */
 		branch = FALSE;
 	}	
 	
@@ -2529,8 +2537,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonBranchMenus");
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextNormal");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL)
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextNormal");
 			}
 			else
 			{
@@ -2540,8 +2549,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonDisabled"); 
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextDisabled");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL) 
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextDisabled");
 			}
 		}
 		else if (pKey->Type == KEYTYPE_BRANCHTOOLBARS)
@@ -2554,8 +2564,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonBranchToolbars"); 
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextNormal");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL)
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextNormal");
 			}
 			else
 			{
@@ -2565,8 +2576,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonDisabled"); 
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextDisabled");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL)
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextDisabled");
 			}
 		}
 		else if (pKey->Type == KEYTYPE_BRANCHGUI)
@@ -2579,8 +2591,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonBranchGUI"); 
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextNormal");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL)
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextNormal");
 			}
 			else
 			{
@@ -2590,8 +2603,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonDisabled"); 
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextDisabled");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL)
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextDisabled");
 			}
 		}
 		else if (pKey->Type == KEYTYPE_BRANCHEDITTEXT)
@@ -2604,8 +2618,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonBranchGUI"); 
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextNormal");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL)
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextNormal");
 			}
 			else
 			{
@@ -2615,8 +2630,9 @@ gboolean gok_keyboard_validate_dynamic_keys (Accessible* pAccessibleForeground)
 					bChanged = TRUE;
 				}
 				gtk_widget_set_name (pKey->pButton, "StyleButtonDisabled"); 
-				g_assert (((GokButton*)pKey->pButton)->pLabel != NULL);
-				gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, "StyleTextDisabled");
+				if (((GokButton*)pKey->pButton)->pLabel != NULL)
+					gtk_widget_set_name (((GokButton*)pKey->pButton)->pLabel, 
+							     "StyleTextDisabled");
 			}
 		}
 		pKey = pKey->pKeyNext;
