@@ -1,6 +1,6 @@
 /* gok-scanner.c
 *
-* Copyright 2002 Sun Microsystems, Inc.,
+* Copyright 2002 Sun Microsystems, Inc.
 * Copyright 2002 University Of Toronto
 *
 * This library is free software; you can redistribute it and/or
@@ -2154,8 +2154,9 @@ void gok_scanner_input_motion (gint *motion_data, gint n_axes)
 	gint x=0, y=0;
 	if ((n_axes >= 2) && pWindow->window) {
 		gint o_x, o_y;
-		x = motion_data[0] - m_OffsetX;
-		y = motion_data[1] - m_OffsetY;
+		gdouble multiplier = gok_data_get_valuator_sensitivity ();
+		x = motion_data[0] * multiplier - m_OffsetX;
+		y = motion_data[1] * multiplier - m_OffsetY;
 		gdk_window_get_origin (pWindow->window, &o_x, &o_y);
 		if (!gok_data_get_drive_corepointer ()) {
 			gok_pointer_clip_to_window (&x, &y, pWindow->window, 
