@@ -41,6 +41,7 @@
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/Xatom.h>
 #include "main.h"
 #include "gok.h"
 #include "gok-word-complete.h"
@@ -324,7 +325,8 @@ gint gok_main_open(gint argc, gchar *argv[])
 	gok_sound_initialize();
 
 	/* create the main window */
-	m_pWindowMain = gok_main_create_window ();
+	m_pWindowMain = gok_main_create_window (
+		gok_data_get_dock_type () != GOK_DOCK_NONE);
 	if (m_pWindowMain == NULL)
 	{
 		gok_main_display_error (_("Can't create the main GOK window!"));
@@ -527,6 +529,7 @@ void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long ti
         GtkWidget *window = gok_main_get_main_window ();
 	Display *display;
 	GdkWindow *root;
+	gint dx, dy;
 	if (window) 
 	{
 		display = GDK_WINDOW_XDISPLAY (window->window);
@@ -537,17 +540,13 @@ void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long ti
 			mods, timestamp);
 		gok_log_leave ();
 		gok_scanner_input_motion (motion_data, n_axes);
-		
-		/* 
-		 * TODO:
-		 * XWarpPointer is apparently too slow to do every time we get a 
-		 * motion event, so if we want to support "cursor latching" we need 
-		 * to coalesce these events.
-		 */
+
+		dx = motion_data[0] * gok_data_get_valuator_sensitivity ();
+		dy = motion_data[1] * gok_data_get_valuator_sensitivity ();
 		if (gok_data_get_drive_corepointer ()) {
 			XWarpPointer (display, None, 
 				      GDK_WINDOW_XWINDOW (root),
-				      0, 0, 0, 0, motion_data[0], motion_data[1]);
+				      0, 0, 0, 0, dx, dy);
 		}
 	}
 }
@@ -1039,7 +1038,7 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 *
 * returns: A pointer to the window, NULL if it couldn't be created.
 */
-GtkWidget* gok_main_create_window ()
+GtkWidget* gok_main_create_window (gboolean is_dock)
 {
 	GtkWidget *window1;
 	GtkWidget *fixed1;
@@ -1056,7 +1055,7 @@ GtkWidget* gok_main_create_window ()
 				"allow_shrink", TRUE,
 				"border_width", 0,
 				NULL),
-			     "signal::realize", on_window1_realize, NULL,
+			     "signal::realize", on_window1_realize, is_dock,
 			     "signal::destroy", on_window1_destroy, NULL,
 			     NULL);
 
@@ -1429,9 +1428,32 @@ void gok_main_resize_window (GtkWidget* pWindow, gint Width, gint Height)
 		return;
 	}
 	
+	/* let the keyboard know that the resize was caused by us */
+	gok_keyboard_set_ignore_resize (TRUE);
+	
 	/* calculate the upper left corner of the window */
-	left = gok_data_get_keyboard_x() - (Width / 2);
-	top = gok_data_get_keyboard_y() - (Height / 2);
+	switch (gok_data_get_dock_type ()) { /* if we're a dock */
+	case GOK_DOCK_BOTTOM:
+	  top = gdk_screen_height ();
+	  left = 0;
+	  if (gok_data_get_expand ()) {
+		  Width = gdk_screen_width ();
+		  gok_keyboard_set_ignore_resize (FALSE);
+	  }
+	  break;
+	case GOK_DOCK_TOP:
+	  top = 0;
+	  left = 0;
+	  if (gok_data_get_expand ()) {
+		  Width = gdk_screen_width ();
+		  gok_keyboard_set_ignore_resize (FALSE);
+	  }
+	  break;
+	default:
+	  top = gok_data_get_keyboard_y() - (Height / 2);
+	  left = gok_data_get_keyboard_x() - (Width / 2);
+	  break;
+	}
 
 	/* add the frame dimension to the window dimension */
 	/* get the frame location */
@@ -1445,11 +1467,14 @@ void gok_main_resize_window (GtkWidget* pWindow, gint Width, gint Height)
 		frameX = (winX - rectFrame.x);
 		frameY = (winY - rectFrame.y);
 	}
-	else
+	else if (gok_data_get_dock_type () == GOK_DOCK_NONE)
 	{
 		frameX = 5;/* this is usually true */
 		frameY = 22;/* this is usually true */
 	}
+	else {
+	        frameX = frameY = 0;
+	}
 	left -= frameX;
 	top -= frameY;
 	
@@ -1478,9 +1503,6 @@ void gok_main_resize_window (GtkWidget* pWindow, gint Width, gint Height)
 		top = screenY - Height - frameY - (frameX * 2);
 	}
 	
-	/* let the keyboard know that the resize was caused by us */
-	gok_keyboard_set_ignore_resize (TRUE);
-	
 	/* ensure the window is within the given geometry */
 	if (m_bUseGivenGeometry == TRUE)
 	{
@@ -1497,11 +1519,48 @@ void gok_main_resize_window (GtkWidget* pWindow, gint Width, gint Height)
 	/* move/resize the main window */
 	gdk_window_move_resize ((GdkWindow*)pWindow->window, left, top, Width, Height);
 
+	if (gok_data_get_dock_type () != GOK_DOCK_NONE)
+		gok_main_update_struts (Width, Height);
+
 	/* store these values */
 	m_OurResizeWidth = Width;
 	m_OurResizeHeight = Height;
 }
 
+/*
+ * TODO: GtkDoc this!
+ */
+void
+gok_main_update_struts (gint width, gint height)
+{
+	GtkWidget *widget = gok_main_get_main_window ();
+	Atom atom_strut = gdk_x11_get_xatom_by_name ("_NET_WM_STRUT");
+	guint32 struts[4];
+
+	switch (gok_data_get_dock_type () != GOK_DOCK_NONE) {
+	case GOK_DOCK_TOP:
+		struts[0] = struts[2] = struts[3] = 0;
+		struts[1] = height;
+		break;
+	case GOK_DOCK_BOTTOM:
+		struts[0] = struts[1] = struts[2]= 0;
+		struts[3] = height;
+		break;
+	default:
+		return;
+		break;
+	}
+	gdk_error_trap_push ();
+	/* this means that we are responsible for placing ourselves appropriately on screen */
+	if (widget && widget->window)
+		XChangeProperty (GDK_WINDOW_XDISPLAY (widget->window), 
+				 GDK_WINDOW_XWINDOW (widget->window), 
+				 atom_strut,
+				 XA_CARDINAL, 32, PropModeReplace,
+				 (guchar *) &struts, 4);
+	gdk_error_trap_pop ();
+}
+
 /**
 * gok_main_get_our_window_size
 * @pWidth: Pointer to a variable that will hold the window width.
